      const N_CELLS  =  20;
      const RADIUS   =   3;
      const INTERVAL = 100;

      function getCanvasData(name)
      {
         var theCanvas = document.getElementById(name);

         if(theCanvas == null)
            return(null);

         return(
            {
               canvas: theCanvas,
               ctx   : theCanvas.getContext("2d"),
               width : theCanvas.width,
               height: theCanvas.height
            }
         );
      }

      function drawLine(ctx, x0, y0, x1, y1)
      {
         ctx.moveTo(x0, y0);
         ctx.lineTo(x1, y1);
         ctx.stroke();
      }

      function drawGrid(canvasData)
      {
         var size      = Math.min(canvasData.width, canvasData.height);
         var cellSize  = Math.floor(size / N_CELLS);
         var saveStyle = canvasData.ctx.strokeStyle;

         canvasData.ctx.strokeStyle = "#c0c0c0";

         for(var x = cellSize; x < canvasData.width; x += cellSize)
            drawLine(canvasData.ctx, x, 0, x, canvasData.height);

         for(var y = cellSize; y < canvasData.height; y += cellSize)
            drawLine(canvasData.ctx, 0, y, canvasData.width, y);

         canvasData.ctx.beginPath();
         canvasData.ctx.rect(0, 0, canvasData.width, canvasData.height);
         canvasData.ctx.stroke();

         canvasData.ctx.strokeStyle = saveStyle;

         return(cellSize);
      }

      function onEdge(values, row, column)
      {
         return(
            row == 0 || column == 0 ||
               row + 1 == values.length || column + 1 == values[0].length
         );
      }

      function grow(values)
      {
         // Get a list of neighbors of existing "on" cells

         var theList = [];

         for(var i = 0; i < values.length; i ++)
         {
            for(var j = 0; j < values[i].length; j ++)
            {
               if(values[i][j] == true)
               {
                  var list = getNeighbors(values, i, j);

                  for(var k = 0; k < list.length; k ++)
                  {
                     var r = list[k].row;
                     var c = list[k].column;

                     if(!values[r][c] && !onEdge(values, r, c))
                        theList.push(list[k]);
                  }
               }
            }
         }

         // A given empty cell might be a neighbor
         // of more than one "on" cell, so remove
         // any duplicate entries

         var history  = {};
         var filtered = [];

         for(var i = 0; i < theList.length; i ++)
         {
            var s = JSON.stringify(theList[i]);

            if(history[s] == null)
            {
               filtered.push(theList[i]);
               history[s] = true;
            }
         }

         theList = filtered;

         if(theList.length == 0)
         {
            // If no cells are on yet, there won't
            // be any neighbors - just pick the
            // center cell

            var r = Math.floor(values.length / 2);
            var c = Math.floor(values[0].length / 2);

            values[r][c] = true;
         }
         else
         {
            // Choose an entry from 'theList' at random

            var entry = theList[Math.floor(Math.random() * theList.length)];

            values[entry.row][entry.column] = true;
         }
      }

      function getNeighbors(values, row, column)
      {
         var list = [];

         for(var i = -1; i <= 1; i ++)
         {
            for(var j = -1; j <= 1; j ++)
            {
               if(i != 0 || j != 0)
               {
                  var r = row    + i;
                  var c = column + j;

                  if(r >= 0 && r < values.length &&
                     c >= 0 && c < values[r].length)
                  {
                     list.push( { row: r, column: c } );
                  }
               }
            }
         }

         return(list);
      }

      function nukeNeighbors(values, row, column)
      {
         values[row][column] = false;

         var list = getNeighbors(values, row, column);
         
         for(var i = 0; i < list.length; i ++)
         {
            var r = list[i].row;
            var c = list[i].column;

            if(values[r][c])
               nukeNeighbors(values, r, c);
         }
      }

      function countIslands(values)
      {
         var count = 0;

         for(;;)
         {
            var found = false;

            for(var i = 0; !found && i < values.length; i ++)
            {
               for(var j = 0; !found && j < values[i].length; j ++)
               {
                  if(values[i][j])
                  {
                     found = true;
                     nukeNeighbors(values, i, j);
                  }
               }
            }

            if(!found)
               break;

            count++;
         }

         return(count);
      }

      function copyValues(values)
      {
         return(JSON.parse(JSON.stringify(values)));
      }

      function countActive(values)
      {
         var count = 0;

         for(var i = 0; i < values.length; i ++)
            for(var j = 0; j < values[i].length; j ++)
               if(values[i][j])
                  count++;

         return(count);
      }

      function shrink(values)
      {
         var theList = [];

         for(var i = 0; i < values.length; i ++)
         {
            for(var j = 0; j < values[i].length; j ++)
            {
               // Don't turn off this cell if doing so
               // will make our picture less connected

               if(values[i][j])
               {
                  var current = countIslands(copyValues(values));
                  var copy    = copyValues(values);

                  copy[i][j] = false;

                  if(current == countIslands(copy))
                     theList.push( { row: i, column: j } );
               }
            }
         }

         if(theList.length > 0)
         {
            var entry = theList[Math.floor(Math.random() * theList.length)];

            values[entry.row][entry.column] = false;
         }
      }

      function makeValues(size)
      {
         var anArray = [];

         for(var i = 0; i < size; i ++)
         {
            var row = [];

            for(var j = 0; j < size; j ++)
               row.push(false);

            anArray.push(row);
         }

         return(anArray);
      }

      function drawCells(values, cellSize, canvasData)
      {
         for(var i = 0; i < values.length; i ++)
         {
            var y = cellSize * i;

            for(var j = 0; j < values[i].length; j ++)
            {
               var x = cellSize * j;

               if(values[i][j])
               {
                  canvasData.ctx.beginPath();

                  canvasData.ctx.arc(
                     x + cellSize / 2,
                     y + cellSize / 2,
                     RADIUS,
                     0,
                     2 * Math.PI, false
                  );

                  canvasData.ctx.fill();
               }

               var xMid = x + (cellSize / 2);
               var xEnd = x + cellSize;
               var yMid = y + (cellSize / 2);
               var yEnd = y + cellSize;

               // Draw boundary according to case number

               var number = getCaseNumber(values, i, j);

               switch(number)
               {
                  case 1:
                  {
                     drawLine(
                        canvasData.ctx, xMid, yEnd, xEnd, yMid + cellSize
                     );
                  }
                  break;

                  case 2:
                  {
                     drawLine(
                        canvasData.ctx, xEnd, yMid + cellSize,
                        xMid + cellSize, yEnd
                     );
                  }
                  break;

                  case 3:
                  {
                     drawLine(
                        canvasData.ctx, xMid, yEnd, xMid + cellSize, yEnd
                     );
                  }
                  break;

                  case 4:
                  {
                     drawLine(
                        canvasData.ctx, xEnd, yMid, xMid + cellSize, y + cellSize
                     );
                  }
                  break;

                  case 5:
                  {
                     drawLine(
                        canvasData.ctx, xMid, yEnd, xEnd, yMid
                     );

                     drawLine(
                        canvasData.ctx, xEnd, yMid + cellSize,
                        xMid + cellSize, yEnd
                     );
                  }
                  break;

                  case 6:
                  {
                     drawLine(
                        canvasData.ctx, xEnd, yMid, xEnd, yMid + cellSize
                     );
                  }
                  break;

                  case 7:
                  {
                     drawLine(
                        canvasData.ctx, xMid, yEnd, xEnd, yMid
                     );
                  }
                  break;

                  case 8:
                  {
                     drawLine(
                        canvasData.ctx, xMid, yEnd, xEnd, yMid
                     );
                  }
                  break;

                  case 9:
                  {
                     drawLine(
                        canvasData.ctx, xEnd, yMid, xEnd, yMid + cellSize
                     );
                  }
                  break;

                  case 10:
                  {
                     drawLine(
                        canvasData.ctx, xMid, yEnd, xEnd, yMid + cellSize
                     );

                     drawLine(
                        canvasData.ctx, xEnd, yMid, xMid + cellSize, yEnd
                     );
                  }
                  break;

                  case 11:
                  {
                     drawLine(
                        canvasData.ctx, xEnd, yMid, xMid + cellSize, yEnd
                     );
                  }
                  break;

                  case 12:
                  {
                     drawLine(
                        canvasData.ctx, xMid, yEnd, xMid + cellSize, yEnd
                     );
                  }
                  break;

                  case 13:
                  {
                     drawLine(
                        canvasData.ctx, xEnd, yMid + cellSize, xMid + cellSize, yEnd
                     );
                  }
                  break;

                  case 14:
                  {
                     drawLine(
                        canvasData.ctx, xMid, yEnd, xEnd, yMid + cellSize
                     );
                  }
                  break;
               }
            }
         }
      }

      function getCaseNumber(values, row, column)
      {
         var rows    = values.length;
         var columns = values[0].length;

         var number = 0;

         if(column + 1 < columns && row + 1 < rows)
         {
            if(values[row][column])
               number |= 8;

            if(column + 1 < columns && values[row][column + 1])
               number |= 4;

            if(column + 1 < columns && row + 1 < rows && values[row + 1][column + 1])
               number |= 2;

            if(row + 1 < rows && values[row + 1][column])
               number |= 1;
         }

         return(number);
      }

      var G =
      {
         values: makeValues(N_CELLS),
         doGrow: true,
         active: false
      };

      function getOnRatio(values)
      {
         return(countActive(values) / (values.length * values[0].length));
      }

      function render()
      {
         if(G.active)
         {
            var canvasData = getCanvasData("the-canvas");

            canvasData.ctx.clearRect(0, 0, canvasData.width, canvasData.height);

            var cellSize = drawGrid(canvasData);

            if(G.doGrow)
            {
               grow(G.values);

               if(getOnRatio(G.values) >= .5)
                  G.doGrow = false;
            }
            else
            {
               shrink(G.values);

               if(countActive(G.values) < 2)
                  G.doGrow = true;
            }

            drawCells(G.values, cellSize, canvasData);
         }

         scheduleRender();
      }

      function scheduleRender()
      {
         window.setTimeout( function() { render(); }, getInterval());
      }

      function toggleAnimation()
      {
         var button = document.getElementById("the-button");

         button.value = (button.value == "Pause Animation" ?
            "Restart Animation" : "Pause Animation");

         G.active = !G.active;

         if(G.active)
            scheduleRender();
      }

      function getInterval()
      {
         return(document.getElementById("the-interval").value);
      }

